@@ -6,13 +6,18 @@ import asyncio
 import logging
 import pickle
 import ssl
+from typing import Union
 
 import serial
 import serial_asyncio
 
-from velbusaio.const import CACHEDIR, LOAD_TIMEOUT
+from velbusaio.const import LOAD_TIMEOUT
 from velbusaio.handler import PacketHandler
+from velbusaio.helpers import get_cache_dir
 from velbusaio.messages.module_type_request import ModuleTypeRequestMessage
+from velbusaio.messages.set_date import SetDate
+from velbusaio.messages.set_daylight_saving import SetDaylightSaving
+from velbusaio.messages.set_realtime_clock import SetRealtimeClock
 from velbusaio.module import Module
 from velbusaio.parser import VelbusParser
 
@@ -22,7 +27,7 @@ class Velbus:
     A velbus controller
     """
 
-    def __init__(self, dsn):
+    def __init__(self, dsn) -> None:
         self._log = logging.getLogger("velbus")
         self._log.setLevel(logging.DEBUG)
         self._dsn = dsn
@@ -33,8 +38,11 @@ class Velbus:
         self._modules = {}
         self._submodules = []
         self._send_queue = asyncio.Queue()
+        self._tasks = []
 
-    async def add_module(self, addr, typ, data, sub_addr=None, sub_num=None):
+    async def add_module(
+        self, addr: str, typ: str, data: dict, sub_addr=None, sub_num=None
+    ) -> None:
         """
         Add a founc module to the module cache
         """
@@ -48,7 +56,7 @@ class Velbus:
         self._modules[addr].initialize(self.send)
         await self._modules[addr].load()
 
-    async def add_submodules(self, addr, subList):
+    async def add_submodules(self, addr, subList) -> None:
         for sub_num, sub_addr in subList.items():
             if sub_addr == 0xFF:
                 continue
@@ -57,20 +65,20 @@ class Velbus:
             self._modules[sub_addr] = self._modules[addr]
         self._modules[addr].cleanupSubChannels()
 
-    def _load_module_from_cache(self, address):
+    def _load_module_from_cache(self, address) -> Union[None, str]:
         try:
-            with open(f"{CACHEDIR}/{address}.p", "rb") as fl:
+            with open(f"{get_cache_dir()}/{address}.p", "rb") as fl:
                 return pickle.load(fl)
         except OSError:
             pass
 
-    def get_modules(self):
+    def get_modules(self) -> dict:
         """
         Return the module cache
         """
         return self._modules
 
-    def get_module(self, addr):
+    def get_module(self, addr: str) -> Union[None, Module]:
         """
         Get a module on an address
         """
@@ -78,7 +86,7 @@ class Velbus:
             return self._modules[addr]
         return None
 
-    def get_channels(self, addr):
+    def get_channels(self, addr: str) -> Union[None, dict]:
         """
         Get the channels for an address
         """
@@ -86,7 +94,13 @@ class Velbus:
             return (self._modules[addr]).get_channels()
         return None
 
-    async def connect(self):
+    async def stop(self) -> None:
+        for task in self._tasks:
+            task.cancel()
+        self._writer.close()
+        await self._writer.wait_closed()
+
+    async def connect(self, test_connect: bool = False) -> None:
         """
         Connect to the bus and load all the data
         """
@@ -113,31 +127,38 @@ class Velbus:
                 xonxoff=0,
                 rtscts=1,
             )
-
+        if test_connect:
+            return
         # create reader, parser and writer tasks
-        asyncio.Task(self._socket_read_task())
-        asyncio.Task(self._socket_send_task())
-        asyncio.Task(self._parser_task())
+        self._tasks.append(asyncio.Task(self._socket_read_task()))
+        self._tasks.append(asyncio.Task(self._socket_send_task()))
+        self._tasks.append(asyncio.Task(self._parser_task()))
         # scan the bus
-        await self._scan()
+        await self.scan()
 
-    async def _scan(self):
+    async def scan(self) -> None:
         self._handler.scan_started()
         for addr in range(1, 255):
             msg = ModuleTypeRequestMessage(addr)
             await self.send(msg)
         await asyncio.sleep(30)
         self._handler.scan_finished()
+        # calculate how long to wait
+        calc_timeout = len(self._modules) * 30
+        if calc_timeout < LOAD_TIMEOUT:
+            timeout = calc_timeout
+        else:
+            timeout = LOAD_TIMEOUT
         # create a task to wait until we have all modules loaded
         tsk = asyncio.Task(self._check_if_modules_are_loaded())
         try:
-            await asyncio.wait_for(tsk, timeout=LOAD_TIMEOUT)
+            await asyncio.wait_for(tsk, timeout=timeout)
         except asyncio.TimeoutError:
             self._log.error(
                 f"Not all modules are laoded within a timeout of {LOAD_TIMEOUT} seconds, continuing with the loaded modules"
             )
 
-    async def _check_if_modules_are_loaded(self):
+    async def _check_if_modules_are_loaded(self) -> None:
         """
         Task to wait until modules are loaded
         """
@@ -149,16 +170,16 @@ class Velbus:
             if mods_loaded == len(self.get_modules()):
                 self._log.info("All modules loaded")
                 return
-            self._log.warning("Not all modules loaded yet, waiting 20 seconds")
-            await asyncio.sleep(20)
+            self._log.info("Not all modules loaded yet, waiting 30 seconds")
+            await asyncio.sleep(230)
 
-    async def send(self, msg):
+    async def send(self, msg) -> None:
         """
         Send a packet
         """
         await self._send_queue.put(msg)
 
-    async def _socket_send_task(self):
+    async def _socket_send_task(self) -> None:
         """
         Task to send the packet from the queue to the bus
         """
@@ -169,7 +190,7 @@ class Velbus:
             self._writer.write(msg.to_binary())
             await asyncio.sleep(0.11)
 
-    async def _socket_read_task(self):
+    async def _socket_read_task(self) -> None:
         """
         Task to read from a socket and push into a queue
         """
@@ -177,7 +198,7 @@ class Velbus:
             data = await self._reader.read(10)
             self._parser.feed(data)
 
-    async def _parser_task(self):
+    async def _parser_task(self) -> None:
         """
         Task to parser the received queue
         """
@@ -185,7 +206,7 @@ class Velbus:
             packet = await self._parser.wait_for_packet()
             await self._handler.handle(packet)
 
-    def get_all(self, class_name):
+    def get_all(self, class_name: str) -> list:
         lst = []
         for addr, mod in (self.get_modules()).items():
             if addr in self._submodules:
@@ -194,3 +215,11 @@ class Velbus:
                 if class_name in chan.get_categories():
                     lst.append(chan)
         return lst
+
+    async def sync_clock(self) -> None:
+        """
+        This will send all the needed messages to sync the clock
+        """
+        await self.send(SetRealtimeClock())
+        await self.send(SetDate())
+        await self.send(SetDaylightSaving())
